﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using CPMotor_DotNET;

namespace Interpolation
{
    public partial class Form1 : Form
    {
        const int Max_Motor = 3;
        const int Motor_Step = 100;

        byte Slot = 0;
        byte Baud = 5;
        byte Type = 1;
        byte[] Id = new byte[Max_Motor];

        public Form1()
        {
            InitializeComponent();

            combo_slot.SelectedIndex = 0;
            combo_baud.SelectedIndex = 5;
            combo_motortype.SelectedIndex = 1;
            combo_motorid1.SelectedIndex = 0;
            combo_motorid2.SelectedIndex = 1;
            combo_motorid3.SelectedIndex = 2;

            Slot = 0;
            Baud = 5;
            Type = 1;

            Id[0] = 1;
            Id[1] = 2;
            Id[2] = 3;            
        }

        private void combo_slot_SelectedIndexChanged(object sender, EventArgs e)
        {
            Slot = (byte)combo_slot.SelectedIndex;
        }

        private void combo_baud_SelectedIndexChanged(object sender, EventArgs e)
        {
            Baud = (byte)combo_baud.SelectedIndex;
        }

        private void combo_motortype_SelectedIndexChanged(object sender, EventArgs e)
        {
            Type = (byte)combo_motortype.SelectedIndex;
        }

        private void combo_motorid1_SelectedIndexChanged(object sender, EventArgs e)
        {
            Id[0] = (byte)(combo_motorid1.SelectedIndex + 1);
        }

        private void combo_motorid2_SelectedIndexChanged(object sender, EventArgs e)
        {
            Id[1] = (byte)(combo_motorid2.SelectedIndex + 1);
        }

        private void combo_motorid3_SelectedIndexChanged(object sender, EventArgs e)
        {
            Id[2] = (byte)(combo_motorid3.SelectedIndex + 1);
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            ushort ret, i;

            for (i = 0; i < Max_Motor; i++)
            {
                ret = CPMotor.CPMotor_MotionStop(Slot, Id[i], 300);
                if (ret != 0)
                {
                    System.Console.WriteLine("CPMotor_MotionStop " + (i + 1).ToString() + " Error " + ret.ToString());
                }
                else
                {
                    ret = CPMotor.CPMotor_CloseMotor(Slot, Id[i]);
                    if (ret != 0)
                    {
                        System.Console.WriteLine("CPMotor_CloseMotor " + (i + 1).ToString() + " Error " + ret.ToString());
                    }
                }
            }

            ret = CPMotor.CPMotor_ShutdownMaster(Slot);
            if (ret != 0)
            {
                System.Console.WriteLine("CPMotor_ShutdownMaster Error " + ret.ToString());
            }
        }

        private void btn_initial_Click(object sender, EventArgs e)
        {
            ushort ret, i;
            byte err_cnt = 0;
            int[] cur_position = new int[Max_Motor];

            // Initial I-8123W and Motor Device
            ret = CPMotor.CPMotor_InitMaster(CPMotor.MASTERMODULE.MASTER_MODULE_PISOCPM100U, Slot, (CPMotor.BAUDRATE)Baud);
            if (ret != 0)
            {
                MessageBox.Show("CPMotor_InitMaster Error " + ret.ToString());
            }
            else
            {
                for (i = 0; i < Max_Motor; i++)
                {
                    ret = CPMotor.CPMotor_AddMotor(Slot, Id[i], 1, 500, (CPMotor.MOTOR_MANUFACTURER)Type);
                    if (ret != 0)
                    {
                        MessageBox.Show("CPMotor_AddMotor " + (i + 1).ToString() + " Error " + ret.ToString());
                        err_cnt++;
                    }
                    else
                    {
                        ret = CPMotor.CPMotor_InitMotor(Slot, Id[i], 100);
                        if (ret != 0)
                        {
                            MessageBox.Show("CPMotor_InitMotor " + (i + 1).ToString() + " Error " + ret.ToString());
                            err_cnt++;
                        }
                        else
                        {
                            // Call CPMotor_IPInitial to into Interpolation mode
                            ret = CPMotor.CPMotor_IPInitial(Slot, Id[i]);
                            if (ret != 0)
                            {
                                MessageBox.Show("CPMotor_IPInitial " + (i + 1).ToString() + " Error " + ret.ToString());
                                err_cnt++;
                            }
                        }
                    }
                }

                if (err_cnt != 0) return;

                MessageBox.Show("AddMotor and InitMotor OK");

                btn_initial.Enabled = false;

                combo_slot.Enabled = false;
                combo_baud.Enabled = false;
                combo_motortype.Enabled = false;
                combo_motorid1.Enabled = false;
                combo_motorid2.Enabled = false;
                combo_motorid3.Enabled = false;

                bar_wpos1.Enabled = true;
                bar_wpos2.Enabled = true;
                bar_wpos3.Enabled = true;

                System.Threading.Thread.Sleep(100);

                timer1.Enabled = true;
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            ushort ret, i;
            byte currmode = 0;
            byte isservoready = 0;
            byte isfault = 0;
            byte iswarning = 0;
            byte isdone = 0;
            ushort statusword = 0;
            int[] cur_position = new int[Max_Motor];

            for (i = 0; i < Max_Motor; i++)
            {
                // Check Motor Status
                ret = CPMotor.CPMotor_MotorStatus(Slot, Id[i], ref currmode, ref isservoready, ref isfault, ref iswarning, ref isdone, ref statusword);
                if (ret == 0)
                {
                    if (i == 0)
                    {
                        if (isfault != 0) lab_status1.Text = "Motor 1 Fault";
                        else if (iswarning != 0) lab_status1.Text = "Motor 1 Warning";
                        else if (isdone != 0) lab_status1.Text = "Motor 1 Done";
                    }
                    if (i == 1)
                    {
                        if (isfault != 0) lab_status2.Text = "Motor 2 Fault";
                        else if (iswarning != 0) lab_status2.Text = "Motor 2 Warning";
                        else if (isdone != 0) lab_status2.Text = "Motor 2 Done";
                    }
                    if (i == 2)
                    {
                        if (isfault != 0) lab_status3.Text = "Motor 3 Fault";
                        else if (iswarning != 0) lab_status3.Text = "Motor 3 Warning";
                        else if (isdone != 0) lab_status3.Text = "Motor 3 Done";
                    }
                }
            }

            ////////// Get Current Position and Set to Silder /////////////
            for (i = 0; i < Max_Motor; i++)
            {
                ret = CPMotor.CPMotor_PP_CurrentPosition(Slot, Id[i], ref cur_position[i]);
                if (ret == 0)
                {
                    if (i == 0)
                    {
                        bar_wpos1.Value = (cur_position[0] / 65536);
                        text_wpos1.Text = cur_position[0].ToString();
                    }
                    if (i == 1)
                    {
                        bar_wpos2.Value = (cur_position[1] / 65536);
                        text_wpos2.Text = cur_position[1].ToString();
                    }
                    if (i == 2)
                    {
                        bar_wpos3.Value = (cur_position[2] / 65536);
                        text_wpos3.Text = cur_position[2].ToString();
                    }
                }
            }
        }

        private void btn_move_Click(object sender, EventArgs e)
        {
            ushort ret, i, j;
            byte[] abs = new byte[Max_Motor];
            int[] path = new int[Max_Motor * Motor_Step];

            // Select Relative Path
            abs[0] = 0;
            abs[1] = 0;
            abs[2] = 0;
            
            // Motor 0: Move 10 Pulse per Step
            // Motor 1: Move 20 Pulse per Step
            // Motor 2: Move 30 Pulse per Step
            for (j = 0; j < Max_Motor; j++) 
            {
                for (i = 0; i < Motor_Step; i++)
                {
                    path[(j * Motor_Step) + i] = 10 * (j + 1);
                }
            }

            // Interpolation Move Start
            ret = CPMotor.CPMotor_IP_PathMove(Slot, Max_Motor, Id, abs, Motor_Step, path, 0);
            if (ret != 0)
            {
                MessageBox.Show("CPMotor_IP_PathMove Error " + ret.ToString());
                return;
            }
        }
    }
}